#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    sampleRate = 44100;
    bufferSize = 512;
    
    gui.setup();
    // add ofParameters to "gui" panel
    gui.add(carrierFreq.set("carrier frequency", 440, 20, 1000));
    gui.add(modFreq.set("mod frequency", 100, 0, 500));
    gui.add(modIndex.set("mod index", 100, 0, 500));
    gui.add(gain.set("gain", 0.2, 0, 1));
    
    
    gui.add(cutoff.set("cutoff", 1000, 1, 8000));
    gui.add(resonance.set("resonance", 45, 0, 100));
    
    gui.add(modIndex2.set("mod2 index",0,0,500));
    gui.add(modFreq2.set("mod2 Freq",0,0,500));
    
    // switch on the DAC - needs to happen at the end of setup
    ofSoundStreamSetup(2,2,this, sampleRate, bufferSize, 4);
}

//--------------------------------------------------------------
void ofApp::update(){
    
}

//--------------------------------------------------------------
void ofApp::draw(){
    ofBackground(255);
    gui.draw();
    drawWaveform();
}

//--------------------------------------------------------------
void ofApp::audioOut(float * output, int bufferSize, int nChannels) {
    for (int i = 0; i < bufferSize; i++) {
        
        // modulator
        modOUT = mod.square(modFreq) * modIndex;
        modOUT2=mod2.triangle(modFreq2)*modIndex2;
        // carrier
        oscOUT = carrier.sinewave(carrierFreq+modOUT+modOUT2);
        //lowpass filter
        filterOUT = lowpass.lores(oscOUT, cutoff, resonance);
        
        mix = filterOUT * gain;
        
        output[i*nChannels    ] = mix; // left channel
        output[i*nChannels+1  ] = mix; // right channel
        
        soundBuffer[i] = mix;
    }
}

//--------------------------------------------------------------
void ofApp::drawWaveform() {
    ofNoFill();
    ofSetColor(50);
    ofSetLineWidth(5);
    ofBeginShape();
    for(int i = 0; i < bufferSize; i++) {
        float x = ofMap(i, 0, bufferSize, 0, ofGetWidth());
        if(pause == false) {
            ofVertex(x, ofGetHeight()*.5 + soundBuffer[i] * 200);
            pauseBuffer[i] = soundBuffer[i];
        } else
            ofVertex(x, ofGetHeight()*.5 + pauseBuffer[i] * 200);
    }
    ofEndShape();
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    if(key == 32) pause = !pause;
    
    if(key == 'r'){
        randomizeParams();
    }
}

void ofApp::randomizeParams(){
    carrierFreq.set(ofRandom(carrierFreq.getMin(),carrierFreq.getMax()));
}




